//
//  GameViewController.m
//  PhysicsDemo
//
//  Created by Christoph Wick on 25/09/14.
//  Copyright (c) 2014 Christoph Wick, i4innovation GmbH. All rights reserved.
//

#import "GameViewController.h"

@implementation GameViewController

-(void)awakeFromNib
{
	// create a new scene
	SCNScene *scene = [SCNScene scene];
	
	// create and add a camera to the scene
	SCNNode *cameraNode = [SCNNode node];
	cameraNode.position = SCNVector3Make(0.0, 30.0, 50.0);
	cameraNode.rotation = SCNVector4Make(-1.0, 0.0, 0.0, M_PI/7);
	cameraNode.camera = [SCNCamera camera];
	cameraNode.camera.xFov = 75.0;
	[scene.rootNode addChildNode:cameraNode];
	
	// create and add a light to the scene
	SCNNode *lightNode = [SCNNode node];
	lightNode.light = [SCNLight light];
	lightNode.light.type = SCNLightTypeSpot;
	lightNode.light.spotOuterAngle = 100;
	lightNode.position = SCNVector3Make(0, 20, 20);
	lightNode.rotation = SCNVector4Make(-1.0, 0.0, 0.0, M_PI_4);
	lightNode.light.castsShadow = YES;
	lightNode.light.shadowRadius = 3.0;
	[scene.rootNode addChildNode:lightNode];
	
	// create and add an ambient light to the scene
	SCNNode *ambientLightNode = [SCNNode node];
	ambientLightNode.light = [SCNLight light];
	ambientLightNode.light.type = SCNLightTypeAmbient;
	ambientLightNode.light.color = [NSColor lightGrayColor];
	[scene.rootNode addChildNode:ambientLightNode];
	
	SCNFloor *floorGeometry = [SCNFloor floor];
	floorGeometry.reflectivity = 0.01;
	floorGeometry.firstMaterial.diffuse.contents = [NSImage imageNamed:@"Grid.png"];
	SCNNode *floorNode = [SCNNode nodeWithGeometry:floorGeometry];
	floorNode.physicsBody = [SCNPhysicsBody staticBody];
	[scene.rootNode addChildNode:floorNode];
	

	// create a "more balls" text node but place it somewhere where it's not yet seen by the default camera
	SCNNode *textPosNode = [SCNNode node];
	textPosNode.position = SCNVector3Make(4.0, 30.0, -5.0);
	textPosNode.rotation = SCNVector4Make(1.0, 0.0, 0.0, -M_PI/8);
	textPosNode.scale = SCNVector3Make(0.02, 0.02, 0.02);
	SCNNode *textNode = [self splittedStylizedTextWithString:@"more balls"];
	[textPosNode addChildNode:textNode];
	
	textPosNode.physicsBody = [SCNPhysicsBody kinematicBody];
	textPosNode.name = @"moreBallsNode";
	[scene.rootNode addChildNode:textPosNode];
	
	
	// set the scene to the view
	scene.physicsWorld.speed = 4.0;
	self.gameView.scene = scene;
	
	// allows the user to manipulate the camera
	self.gameView.allowsCameraControl = YES;
	
	// show statistics such as fps and timing information
	//	self.gameView.showsStatistics = YES;
	self.gameView.showsStatistics = YES;
	
	// configure the view
	self.gameView.backgroundColor = [NSColor blackColor];
}

// Takes a string an creates a node hierarchy where each letter is an independent geometry that is animated
// method heavily inspired by Apple's scene kit demo
- (SCNNode *)splittedStylizedTextWithString:(NSString *)string {
	
	SCNNode *textNode = [SCNNode node];
	SCNMaterial *frontMaterial = [self textFrontMaterial];
	SCNMaterial *border = [self textSideAndChamferMaterial];
	
	// Current x position of the next letter to add
	CGFloat positionX = 0;
	
	// For each letter
	for (NSUInteger i = 0; i < [string length]; i++) {
		
		SCNNode *letterNode = [SCNNode node];
		NSString *letterString = [string substringWithRange:NSMakeRange(i, 1)];
		SCNText *text = [SCNText textWithString:[self attributedStringWithString:letterString] extrusionDepth:50.0];
		
		text.chamferRadius = 3.0;
		text.chamferProfile = [self textChamferProfile];
		
		// use a different material for the "heart" character
		SCNMaterial *finalFrontMaterial = frontMaterial;
		if (i == 1) {
			finalFrontMaterial = [finalFrontMaterial copy];
			finalFrontMaterial.diffuse.contents = [NSColor orangeColor];
			finalFrontMaterial.reflective.contents = [NSColor blackColor];
			letterNode.scale = SCNVector3Make(1.1, 1.1, 1.0);
		}
		
		text.materials = @[finalFrontMaterial, finalFrontMaterial, border, border, border];
		
		letterNode.geometry = text;
		[textNode addChildNode:letterNode];
		
		// measure the letter we just added to update the position
		SCNVector3 min, max;
		max = SCNVector3Make(0, 0, 0);
		min = SCNVector3Make(0, 0, 0);
		if ([letterNode getBoundingBoxMin:&min max:&max]) {
			letterNode.position = SCNVector3Make(positionX - min.x + ( max.x + min.x) * 0.5, -min.y, 0);
			positionX += max.x;
		}
		else{
			// if we have no bounding box, it is probably because of the "space" character.
			// In that case, move to the right a little bit.
			positionX += 50.0;
		}
		
		// Place the pivot at the center of the letter so that the rotation animation looks good
		letterNode.pivot = SCNMatrix4MakeTranslation((max.x + min.x) * 0.5, 0, 0);
		
		// Animate the letters
		CAKeyframeAnimation *animation = [CAKeyframeAnimation animationWithKeyPath:@"rotation"];
		animation.duration = 4.0;
		animation.keyTimes = @[@0.0, @0.3, @1.0];
		animation.values = @[[NSValue valueWithSCNVector4:SCNVector4Make(0, 1, 0, 0)],
							 [NSValue valueWithSCNVector4:SCNVector4Make(0, 1, 0, M_PI * 2)],
							 [NSValue valueWithSCNVector4:SCNVector4Make(0, 1, 0, M_PI * 2)]];
		CAMediaTimingFunction *timingFunction = [CAMediaTimingFunction functionWithName:kCAMediaTimingFunctionEaseInEaseOut];
		animation.timingFunctions = @[timingFunction, timingFunction, timingFunction];
		animation.repeatCount = FLT_MAX;
		animation.beginTime = CACurrentMediaTime() + 1.0 + i * 0.2; // desynchronize animations
		[letterNode addAnimation:animation forKey:nil];
	}
	
	return textNode;
}
- (NSAttributedString *)attributedStringWithString:(NSString *)string {
	NSFont *font = [NSFont fontWithName:@"Avenir Next Heavy" size:288];
	NSDictionary *attributes = @{ NSFontAttributeName : font };
	return [[NSMutableAttributedString alloc] initWithString:string attributes:attributes];
}

- (SCNMaterial *)textFrontMaterial {
	SCNMaterial *material = [SCNMaterial material];
	material.diffuse.contents = [NSColor blackColor];
	material.reflective.intensity = 0.5;
	return material;
}

- (SCNMaterial *)textSideAndChamferMaterial {
	SCNMaterial *material = [SCNMaterial material];
	material.diffuse.contents = [NSColor whiteColor];
	material.reflective.intensity = 0.4;
	return material;
}

- (NSBezierPath *)textChamferProfile {
	NSBezierPath *profile = [NSBezierPath bezierPath];
	[profile moveToPoint:NSMakePoint(0, 1)];
	[profile lineToPoint:NSMakePoint(1.5, 1)];
	[profile lineToPoint:NSMakePoint(1.5, 0)];
	[profile lineToPoint:NSMakePoint(1, 0)];
	return profile;
}


@end
